import os
import sys
import streamlit as st
import pandas as pd
import plotly.express as px

# Add the project root directory to the system path
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '../'))
sys.path.append(project_root)

# Import the settings object
from config.settings import settings

# Load data
data_file_path = os.path.join(settings['data_path'], 'processed_data.csv')
if os.path.exists(data_file_path):
    data = pd.read_csv(data_file_path)
else:
    st.error(f"{data_file_path} not found.")
    st.stop()

# Title and description
st.title("Trading Dashboard")
st.write("This dashboard displays trading data and performance metrics.")

# Summary statistics
st.subheader("Summary Statistics")
st.write(data.describe())

# Plot closing price
st.subheader("Closing Price Over Time")
fig = px.line(data, x='Date', y='Close', title='Closing Price Over Time')
st.plotly_chart(fig)

# Plot trading signals
st.subheader("Trading Signals")
buy_signals = data[data['signal'] == 1]
sell_signals = data[data['signal'] == -1]

fig = px.scatter(data, x='Date', y='Close', title='Trading Signals', color='signal',
                 color_discrete_map={1: 'green', -1: 'red', 0: 'blue'})
fig.add_scatter(x=buy_signals['Date'], y=buy_signals['Close'], mode='markers', name='Buy', marker=dict(color='green', symbol='triangle-up'))
fig.add_scatter(x=sell_signals['Date'], y=sell_signals['Close'], mode='markers', name='Sell', marker=dict(color='red', symbol='triangle-down'))
st.plotly_chart(fig)

# Show raw data
st.subheader("Raw Data")
st.write(data)

# Interactive filters
st.sidebar.header("Filters")
selected_date_range = st.sidebar.date_input("Select date range", [])
selected_stock_code = st.sidebar.selectbox("Select stock code", options=data['stock_code'].unique())

filtered_data = data.copy()
if selected_date_range:
    start_date, end_date = selected_date_range
    filtered_data = filtered_data[(filtered_data['Date'] >= pd.to_datetime(start_date)) & (filtered_data['Date'] <= pd.to_datetime(end_date))]
if selected_stock_code:
    filtered_data = filtered_data[filtered_data['stock_code'] == selected_stock_code]

st.subheader("Filtered Data")
st.write(filtered_data)

# Feature correlations
st.subheader("Feature Correlations")
corr = data.corr()
fig = px.imshow(corr, text_auto=True, title='Feature Correlations')
st.plotly_chart(fig)

# Real-time updates (for illustration purposes)
st.sidebar.header("Real-time Updates")
if st.sidebar.button("Update Data"):
    # Add your data update logic here
    st.sidebar.write("Data updated!")
