import os
import requests
import logging
import json
from datetime import datetime, timedelta
import pandas as pd
from dotenv import load_dotenv
import time

# Load environment variables from .env file located in the config directory
config_dir = 'C:\\Users\\user\\projects\\st\\config'
load_dotenv(dotenv_path=os.path.join(config_dir, '.env'))

settings = {
    'data_path': os.getenv('DATA_PATH'),
    'model_path': os.getenv('MODEL_PATH'),
    'approval_key': os.getenv('APPROVAL_KEY'),
    'max_detected_stocks': int(os.getenv('MAX_DETECTED_STOCKS', 3000)),
    'data_fetch_interval': int(os.getenv('DATA_FETCH_INTERVAL', 60)),
    'strategy_name': os.getenv('STRATEGY_NAME'),
    'token_path': os.getenv('TOKEN_PATH'),
    'appkey': os.getenv('APPKEY'),
    'appsecret': os.getenv('APPSECRET'),
    'user_id': os.getenv('USER_ID'),
    'trading_url': os.getenv('TRADING_URL'),
    'agent': os.getenv('AGENT'),
    'acct_stock': os.getenv('ACCT_STOCK'),
    'prod': os.getenv('PROD'),
    'cano': os.getenv('CANO')
}

# Load additional settings from config.json
config_json_path = os.path.join(config_dir, 'config.json')
if os.path.exists(config_json_path):
    with open(config_json_path, 'r') as f:
        config_settings = json.load(f)
        settings.update(config_settings)
else:
    raise FileNotFoundError(f"{config_json_path} not found.")

logging.basicConfig(level=logging.INFO)

# Token handling functions
def get_token_data():
    token_path = settings['token_path']
    try:
        with open(token_path, 'r') as file:
            return json.load(file)
    except (FileNotFoundError, json.JSONDecodeError) as e:
        logging.error(f"Error reading token file: {e}")
        return None

def save_token_data(token_data):
    token_path = settings['token_path']
    os.makedirs(os.path.dirname(token_path), exist_ok=True)
    with open(token_path, 'w') as file:
        json.dump(token_data, file)

def is_token_valid(token_data):
    if not token_data or 'access_token' not in token_data:
        return False
    expiration_str = token_data.get('expiration')
    if not expiration_str:
        return False
    expiration = datetime.strptime(expiration_str, "%Y-%m-%d %H:%M:%S")
    return expiration > datetime.now()

def get_valid_token():
    token_data = get_token_data()
    if is_token_valid(token_data):
        logging.info("Using existing token")
        return token_data['access_token']
    else:
        logging.info("Token expired or invalid, refreshing...")
        return refresh_token()

def refresh_token(retries=3, delay=60):
    url = f"{settings['trading_url']}/oauth2/tokenP"
    data = {
        'grant_type': 'client_credentials',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret']
    }
    headers = {
        'Content-Type': 'application/json'
    }
    
    for attempt in range(retries):
        try:
            logging.info("Requesting new token...")
            response = requests.post(url, headers=headers, json=data)
            if response.status_code == 200:
                response_data = response.json()
                new_token = response_data.get('access_token')
                expires_in = response_data.get('access_token_token_expired')
                expiration = datetime.now() + timedelta(seconds=int(expires_in) if expires_in else 3600)
                token_data = {'access_token': new_token, 'expiration': expiration.strftime("%Y-%m-%d %H:%M:%S")}
                save_token_data(token_data)
                logging.info(f"New token saved. Expiration time: {expiration}")
                return new_token
            else:
                logging.error(f"Error refreshing token: {response.text}")
                if "EGW00133" in response.text:
                    logging.info(f"Retrying in {delay} seconds...")
                    time.sleep(delay)
                else:
                    return None
        except requests.exceptions.RequestException as e:
            logging.error(f"Request exception occurred while refreshing token: {e}")
            return None
    return None

def get_api_response(url, headers, params):
    try:
        response = requests.get(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        logging.error(f"Request exception occurred: {e}")
        return None

def fetch_stock_data(stock_code, appkey, appsecret, access_token, start_date, end_date):
    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/quotations/inquire-daily-itemchartprice"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': appkey,
        'appsecret': appsecret,
        'tr_id': 'FHKST03010100'
    }
    params = {
        'FID_COND_MRKT_DIV_CODE': 'J',
        'FID_INPUT_ISCD': stock_code,
        'FID_INPUT_DATE_1': start_date.strftime("%Y%m%d"),
        'FID_INPUT_DATE_2': end_date.strftime("%Y%m%d"),
        'FID_PERIOD_DIV_CODE': 'D',
        'FID_ORG_ADJ_PRC': '0'
    }

    return get_api_response(url, headers, params)

def get_search_conditions():
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/quotations/psearch-title"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900300'
    }
    params = {
        'user_id': settings['user_id']
    }

    logging.info(f"Request URL: {url}")
    logging.info(f"Request Headers: {headers}")
    logging.info(f"Request Params: {params}")

    search_conditions = []
    attempt_count = 0
    max_attempts = 10
    next_page_token = None

    while attempt_count < max_attempts:
        if next_page_token:
            params['next_key'] = next_page_token

        response = get_api_response(url, headers, params)
        logging.info(f"API Response: {response}")

        if response and 'output2' in response:
            search_conditions.extend(response['output2'])
            break
        elif response and response.get('msg_cd') == 'MCA05762':
            logging.info("Handling pagination.")
            attempt_count += 1
            next_page_token = response.get('next_key')
            if not next_page_token:
                logging.error("Expected next_key for continuation but not found in response.")
                break
        else:
            logging.error(f"Unexpected response: {response}")
            break

    if attempt_count >= max_attempts:
        logging.error("Reached maximum pagination attempts. Exiting.")
    return search_conditions if search_conditions else None

def get_search_results(seq):
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/quotations/psearch-result"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900400'
    }
    params = {
        'user_id': settings['user_id'],
        'seq': seq
    }

    response = get_api_response(url, headers, params)
    logging.info(f"API Response: {response}")

    if isinstance(response, dict) and 'output2' in response:
        return response['output2']
    else:
        logging.error(f"Unexpected response format or missing 'output2': {response}")
        return None

def save_data_to_csv(data, stock_code):
    file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")
    os.makedirs(os.path.dirname(file_path), exist_ok=True)

    if 'output1' in data:
        output1_data = {
            'stck_prdy_clpr': data['output1'].get('stck_prdy_clpr'),
            'acml_tr_pbmn': data['output1'].get('acml_tr_pbmn'),
            'stck_mxpr': data['output1'].get('stck_mxpr'),
            'stck_llam': data['output1'].get('stck_llam'),
            'stck_oprc': data['output1'].get('stck_oprc'),
            'stck_hgpr': data['output1'].get('stck_hgpr'),
            'stck_lwpr': data['output1'].get('stck_lwpr'),
            'stck_prpr': data['output1'].get('stck_prpr'),
            'stck_fcam': data['output1'].get('stck_fcam'),
            'lstn_stcn': data['output1'].get('lstn_stcn'),
            'cpfn': data['output1'].get('cpfn'),
            'hts_avls': data['output1'].get('hts_avls'),
            'per': data['output1'].get('per'),
            'eps': data['output1'].get('eps'),
            'pbr': data['output1'].get('pbr'),
            'itewhol_loan_rmnd_ratem name': data['output1'].get('itewhol_loan_rmnd_ratem name')
        }

        df = pd.DataFrame([output1_data])
        df.to_csv(file_path, mode='a', index=False, header=not os.path.exists(file_path))
        logging.info(f"Output1 data saved to {file_path}")

    if 'output2' in data:
        for item in data['output2']:
            output2_data = {
                'stck_clpr': item.get('stck_clpr'),
                'acml_tr_pbmn': item.get('acml_tr_pbmn'),
                'stck_oprc': item.get('stck_oprc'),
                'stck_hgpr': item.get('stck_hgpr'),
                'stck_lwpr': item.get('stck_lwpr'),
                'prdy_vrss': item.get('prdy_vrss'),
                'prdy_vrss_sign': item.get('prdy_vrss_sign'),
                'prtt_rate': item.get('prtt_rate'),
                'flng_cls_code': item.get('flng_cls_code'),
                'stck_bsop_date': item.get('stck_bsop_date'),
                'revl_issu_reas': item.get('revl_issu_reas'),
                'mod_yn': item.get('mod_yn')
            }

            df = pd.DataFrame([output2_data])
            df.to_csv(file_path, mode='a', index=False, header=not os.path.exists(file_path))
        logging.info(f"Output2 data appended to {file_path}")
    else:
        logging.error("Data format not recognized for saving to CSV.")

def get_stock_codes_and_names():
    search_conditions = get_search_conditions()
    if not search_conditions:
        logging.error("No search conditions found.")
        return []

    seq = search_conditions[0]['seq']
    search_results = get_search_results(seq)
    if not search_results:
        logging.error("No search results found.")
        return []

    stock_codes_and_names = [(result['code'], result['name']) for result in search_results if isinstance(result, dict) and result.get('code') and result.get('name')]
    logging.info(f"Detected stock codes and names: {stock_codes_and_names}")
    save_stock_codes_to_csv(stock_codes_and_names)
    return stock_codes_and_names

def save_stock_codes_to_csv(stock_codes_and_names):
    file_path = os.path.join(settings['data_path'], 'stock_codes.csv')
    os.makedirs(os.path.dirname(file_path), exist_ok=True)
    
    df = pd.DataFrame(stock_codes_and_names, columns=['code', 'name'])
    df.to_csv(file_path, index=False)
    logging.info(f"Stock codes saved to {file_path}")

def fetch_and_save_data(stock_codes_and_names):
    token_data = get_token_data()
    access_token = token_data['access_token'] if token_data else None

    today = datetime.now().date()
    ten_years_ago = today - timedelta(days=365 * 10)
    date_range = pd.date_range(end=today, start=ten_years_ago, freq='MS')

    for start_date, end_date in zip(date_range, date_range[1:]):
        logging.info(f"Fetching data for date range: {start_date} to {end_date}")
        for stock_code, stock_name in stock_codes_and_names[:settings['max_detected_stocks']]:
            data = fetch_stock_data(stock_code, settings['appkey'], settings['appsecret'], access_token, start_date, end_date)
            if data:
                save_data_to_csv(data, stock_code)
            else:
                logging.error(f"No data fetched for stock: {stock_code}")

            time.sleep(0.05)  # Delay of 0.05 seconds between API requests

def main():
    stock_codes_and_names = get_stock_codes_and_names()
    if not stock_codes_and_names:
        logging.error("No stock codes detected.")
        return

    fetch_and_save_data(stock_codes_and_names)

if __name__ == "__main__":
    main()

# Functions to be called from main_total.py
def fetch_data_for_detected_stocks(detected_stocks):
    fetch_and_save_data(detected_stocks)
