import pandas as pd
from sklearn.preprocessing import MinMaxScaler

def handle_missing_values(data):
    data.fillna(method='ffill', inplace=True)  # 전방 채우기
    data.fillna(method='bfill', inplace=True)  # 후방 채우기
    return data

def normalize_data(data, exclude_columns=[]):
    scaler = MinMaxScaler(feature_range=(0, 1))
    columns_to_scale = [col for col in data.columns if col not in exclude_columns]
    scaled_data = scaler.fit_transform(data[columns_to_scale])
    scaled_df = pd.DataFrame(scaled_data, columns=columns_to_scale, index=data.index)
    for col in exclude_columns:
        scaled_df[col] = data[col]
    return scaled_df, scaler

def add_technical_indicators(data):
    data['MA_20'] = data['Close'].rolling(window=20).mean()
    data['MA_50'] = data['Close'].rolling(window=50).mean()
    data['BB_upper'] = data['MA_20'] + 2 * data['Close'].rolling(window=20).std()
    data['BB_lower'] = data['MA_20'] - 2 * data['Close'].rolling(window=20).std()

    # Moving Average Convergence Divergence (MACD)
    data['MACD'] = data['Close'].ewm(span=12, adjust=False).mean() - data['Close'].ewm(span=26, adjust=False).mean()
    data['MACD_signal'] = data['MACD'].ewm(span=9, adjust=False).mean()

    # Relative Strength Index (RSI)
    delta = data['Close'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=14).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=14).mean()
    rs = gain / loss
    data['RSI'] = 100 - (100 / (1 + rs))

    # Stochastic Oscillator
    low_14 = data['Low'].rolling(window=14).min()
    high_14 = data['High'].rolling(window=14).max()
    data['%K'] = (data['Close'] - low_14) * 100 / (high_14 - low_14)
    data['%D'] = data['%K'].rolling(window=3).mean()

    # Momentum
    data['Momentum'] = data['Close'] - data['Close'].shift(10)

    # Average True Range (ATR)
    high_low = data['High'] - data['Low']
    high_close = (data['High'] - data['Close'].shift()).abs()
    low_close = (data['Low'] - data['Close'].shift()).abs()
    ranges = pd.concat([high_low, high_close, low_close], axis=1)
    true_range = ranges.max(axis=1)
    data['ATR'] = true_range.rolling(window=14).mean()

    # Volume Weighted Average Price (VWAP)
    data['VWAP'] = (data['Close'] * data['Volume']).cumsum() / data['Volume'].cumsum()

    data.dropna(inplace=True)  # 기술 지표 추가 후 결측값 제거
    return data

def split_data(data, target_col='Close'):
    from sklearn.model_selection import train_test_split
    feature_columns = [col for col in data.columns if col != target_col]
    X = data[feature_columns]
    y = data[target_col]
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, shuffle=False)
    return (X_train, X_test, y_train, y_test)

def preprocess_data(stock_data, financial_data, news_data, economic_data):
    integrated_data = stock_data.copy()
    integrated_data = pd.merge(integrated_data, financial_data, how='left', left_index=True, right_index=True)
    integrated_data = handle_missing_values(integrated_data)
    integrated_data, scaler = normalize_data(integrated_data, exclude_columns=['Close'])
    integrated_data = add_technical_indicators(integrated_data)
    return split_data(integrated_data), scaler

if __name__ == '__main__':
    from data_collection import update_data
    stock_data, news_data, financial_data, economic_data = update_data()
    (X_train, X_test, y_train, y_test), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)
    print(X_train.head())
    print(X_test.head())
def data_processing_function():
    print("data_processing_function called")
