import pandas as pd

def add_technical_indicators(data):
    # Simple Moving Averages
    data['MA_20'] = data['Close'].rolling(window=20).mean()
    data['MA_50'] = data['Close'].rolling(window=50).mean()

    # Bollinger Bands
    data['BB_upper'] = data['MA_20'] + 2 * data['Close'].rolling(window=20).std()
    data['BB_lower'] = data['MA_20'] - 2 * data['Close'].rolling(window=20).std()

    # Moving Average Convergence Divergence (MACD)
    data['MACD'] = data['Close'].ewm(span=12, adjust=False).mean() - data['Close'].ewm(span=26, adjust=False).mean()
    data['MACD_signal'] = data['MACD'].ewm(span=9, adjust=False).mean()

    # Relative Strength Index (RSI)
    delta = data['Close'].diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=14).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=14).mean()
    rs = gain / loss
    data['RSI'] = 100 - (100 / (1 + rs))

    # Stochastic Oscillator
    low_14 = data['Low'].rolling(window=14).min()
    high_14 = data['High'].rolling(window=14).max()
    data['%K'] = (data['Close'] - low_14) * 100 / (high_14 - low_14)
    data['%D'] = data['%K'].rolling(window=3).mean()

    # Momentum
    data['Momentum'] = data['Close'] - data['Close'].shift(10)

    # Average True Range (ATR)
    high_low = data['High'] - data['Low']
    high_close = (data['High'] - data['Close'].shift()).abs()
    low_close = (data['Low'] - data['Close'].shift()).abs()
    ranges = pd.concat([high_low, high_close, low_close], axis=1)
    true_range = ranges.max(axis=1)
    data['ATR'] = true_range.rolling(window=14).mean()

    # Volume Weighted Average Price (VWAP)
    data['VWAP'] = (data['Close'] * data['Volume']).cumsum() / data['Volume'].cumsum()

    # Drop NA values after adding indicators
    data.dropna(inplace=True)

    return data

def add_features(data):
    data = add_technical_indicators(data)
    return data

if __name__ == '__main__':
    from data_collection import collect_stock_data
    from config import *

    stock_data = collect_stock_data(STOCK_TICKER, START_DATE, END_DATE)
    stock_data = add_features(stock_data)
    print(stock_data.head())
