import sys
import os
import logging
import threading
import time
import pandas as pd

# 현재 스크립트 위치에서 config 경로를 sys.path에 추가
sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'config'))
sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'scripts'))

from data_collection import read_stock_data, check_and_fix_columns
from model_training import train_lstm_model, update_lstm_model, build_lstm_model
from dqn_model import train_dqn_model
from trading_logic import generate_trading_signals, execute_trades
from settings import settings  # settings 불러오기
from news_data_collector import update_news_data  # 뉴스 데이터 수집기 불러오기
from stock_data_collector import get_stock_codes  # 종목 코드 가져오기

# 로깅 설정 추가
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

def initialize():
    logging.info("Initializing system...")
    # Add your system initialization logic here

def integrate_news_data(stock_data, news_data):
    # 종목별 데이터와 뉴스 데이터를 통합하는 로직을 구현합니다
    # 예시로, 뉴스 데이터를 주식 데이터에 병합할 수 있습니다
    news_data['date'] = pd.to_datetime(news_data['date'])
    stock_data['date'] = pd.to_datetime(stock_data['date'])
    return stock_data.merge(news_data, how='left', left_on='date', right_on='date')

# 학습 스레드
def training_thread(data_path=settings['data_path']):
    while True:
        stock_codes = get_stock_codes()  # 종목 코드 가져오기
        for stock_code in stock_codes:
            df = read_stock_data(stock_code)
            if df is None or df.empty:
                continue
            df = check_and_fix_columns(df)

            if len(df) < 10:
                logging.error(f"Data for stock {stock_code} is insufficient for training. Skipping.")
                continue

            # 뉴스 데이터 수집 및 통합
            news_data = update_news_data(stock_code)
            df = integrate_news_data(df, news_data)

            lstm_model_path = os.path.join(settings['model_path'], f"lstm_{stock_code}.h5")
            if os.path.exists(lstm_model_path):
                lstm_model = build_lstm_model((60, 1))
                lstm_model.load_weights(lstm_model_path)
                lstm_model = update_lstm_model(df['Close'], lstm_model)
            else:
                lstm_model = train_lstm_model(df['Close'])
            lstm_model.save(lstm_model_path)

            train_dqn_model(df, stock_code)

        time.sleep(60)  # 1분 간격으로 모델을 재학습

# 거래 스레드
def trading_thread():
    while True:
        stock_codes = get_stock_codes()  # 종목 코드 가져오기
        for stock_code in stock_codes:
            stock_data = read_stock_data(stock_code)
            if stock_data is not None:
                # 뉴스 데이터 수집 및 통합
                news_data = update_news_data(stock_code)
                stock_data = integrate_news_data(stock_data, news_data)
                
                stock_data = generate_trading_signals(stock_data)
                try:
                    execute_trades(stock_data)
                except Exception as e:
                    logging.error(f"Error executing trades for stock {stock_code}: {e}")
            else:
                logging.warning(f"No data found for stock {stock_code}")
        time.sleep(1)  # 1초 간격으로 거래 수행

def start_trading():
    # 스레드 생성
    training = threading.Thread(target=training_thread)
    trading = threading.Thread(target=trading_thread)

    # 스레드 시작
    training.start()
    trading.start()

    # 스레드가 종료될 때까지 대기
    training.join()
    trading.join()

# WebSocket 콜백 함수
def on_close(ws):
    logging.error("Connection to remote host was lost.")

def on_open(ws):
    global buy_signals
    logging.info("Websocket connected")
    # Define or update `buy_signals` if necessary

# 메인 함수
if __name__ == "__main__":
    initialize()
    start_trading()
