import os
import sys
import logging
import threading
import time
from datetime import datetime, timedelta
import schedule
import pandas as pd

# 프로젝트 루트 디렉터리와 관련된 모든 스크립트 디렉터리를 sys.path에 추가합니다.
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
scripts_dir = os.path.join(project_root, 'scripts')
config_dir = os.path.join(project_root, 'config')
data_dir = os.path.join(project_root, 'data')

sys.path.extend([
    os.path.join(scripts_dir, 'data_collection'),
    os.path.join(scripts_dir, 'model'),
    os.path.join(scripts_dir, 'trading'),
    os.path.join(scripts_dir, 'utils'),
    os.path.join(scripts_dir, 'data_processing'),
    os.path.join(scripts_dir, 'api'),
    config_dir  # config 폴더 경로 추가
])

# config 폴더 경로가 제대로 추가되었는지 확인
print("Config directory added to sys.path:", config_dir in sys.path)
print("Current sys.path:", sys.path)

# 동적으로 config.settings 모듈 로드
import settings

settings = settings.settings

# 필요한 모듈 가져오기
from performance_monitoring import PerformanceMonitor
from portfolio_management import PortfolioManager
from error_handler import log_and_notify_error
from websocket_client import connect_to_websocket

# stock_data_collector에서 필요한 함수 가져오기
from stock_data_collector import fetch_stock_data, save_data_to_csv, get_valid_token, get_stock_codes_and_names
from data_collection import read_stock_data  # 여기서 read_stock_data 함수를 가져옵니다

logging.basicConfig(level=logging.INFO)

available_cash = 0
performance_monitor = PerformanceMonitor()
portfolio_manager = PortfolioManager()

def update_balance():
    try:
        from check_balance import get_acct_balance
        global available_cash
        _, df_output2, _ = get_acct_balance()
        if df_output2 is not None and not df_output2.empty:
            try:
                dnca_tot_amt = df_output2.iloc[0]['dnca_tot_amt']
                available_cash = float(dnca_tot_amt)
                logging.info(f"Updated available cash: {available_cash}")
            except KeyError as e:
                log_and_notify_error(f"Key error: {e}")
            except Exception as e:
                log_and_notify_error(f"Error updating balance: {e}")
        else:
            log_and_notify_error("Failed to update balance.")
    except Exception as e:
        log_and_notify_error(f"Unhandled exception in update_balance: {e}")

def schedule_tasks():
    schedule.every(1).minute.do(update_balance)
    while True:
        schedule.run_pending()
        time.sleep(1)

def initialize():
    logging.info("Initializing system...")

def get_stock_codes_from_csv(file_path):
    try:
        if not os.path.exists(file_path):
            logging.info(f"{file_path} not found. Generating stock codes CSV...")
            stock_codes_and_names = get_stock_codes_and_names()
            save_stock_codes_to_csv(stock_codes_and_names)
        df = pd.read_csv(file_path, dtype={'code': str}, usecols=['code'])  # 'code' 열만 읽음
        if df.empty:
            raise ValueError("CSV file is empty.")
        stock_codes = df['code'].tolist()  # 코드명만 추출
        stock_codes = [code.zfill(6) for code in stock_codes if len(code) <= 6]  # 6자리 코드로 맞춤
        if not stock_codes:
            raise ValueError("No valid stock codes found in the file.")
        return stock_codes
    except pd.errors.EmptyDataError:
        log_and_notify_error("CSV file is empty.")
        return []
    except pd.errors.ParserError:
        log_and_notify_error("Error parsing CSV file.")
        return []
    except Exception as e:
        log_and_notify_error(f"Error reading stock codes from CSV file: {e}")
        return []

def main():
    try:
        balance_thread = threading.Thread(target=schedule_tasks)
        balance_thread.daemon = True
        balance_thread.start()

        from data_preprocessing import preprocess_data
        from feature_engineering import add_features
        from reinforcement_learning import train_reinforcement_learning_model
        from live_data_processing import real_time_prediction, update_real_time_model
        from trading_logic import execute_trades

        logging.info("Starting data collection...")
        
        logging.info("Fetching stock codes and names...")
        global stock_codes
        stock_codes_csv_path = os.path.join(data_dir, 'stock_codes.csv')
        stock_codes = get_stock_codes_from_csv(stock_codes_csv_path)
        if not stock_codes:
            log_and_notify_error("No stock codes detected. Exiting process.")
            return
        logging.info(f"Stock codes: {stock_codes}")

        today = datetime.now().date()
        ten_years_ago = today - timedelta(days=365 * 10)
        date_range = pd.date_range(end=today, start=ten_years_ago, freq='MS')

        access_token = get_valid_token()

        for start_date, end_date in zip(date_range, date_range[1:]):
            logging.info(f"Fetching data for date range: {start_date} to {end_date}")
            for stock_code in stock_codes[:settings['max_detected_stocks']]:
                file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")
                if os.path.exists(file_path):
                    logging.info(f"Data for stock {stock_code} already exists. Skipping.")
                    continue

                try:
                    data = fetch_stock_data(stock_code, settings['appkey'], settings['appsecret'], access_token, start_date, end_date)
                    if data:
                        save_data_to_csv(data, stock_code)
                    else:
                        log_and_notify_error(f"No data fetched for stock: {stock_code}")
                except Exception as e:
                    log_and_notify_error(f"Error fetching data for stock {stock_code}: {e}")

                time.sleep(0.05)

        logging.info("Data collection completed.")

        for stock_code in stock_codes:
            logging.info(f"Processing stock: {stock_code}")
            try:
                stock_data = read_stock_data(stock_code)
                if stock_data is not None:
                    logging.info(f"Stock data for {stock_code} loaded successfully.")
                    
                    (train_data, test_data), scaler = preprocess_data(stock_data, None, None, None)
                    stock_data = add_features(stock_data)

                    rl_model = train_reinforcement_learning_model(stock_data)

                    real_time_predictions = real_time_prediction(rl_model, scaler, stock_code)
                    if real_time_predictions is not None:
                        logging.info(f"Real-time predictions for {stock_code}: {real_time_predictions}")
                    else:
                        log_and_notify_error("Failed to fetch real-time predictions.")

                    model = update_real_time_model(rl_model, scaler, stock_code)

                    execute_trades(stock_data)
                else:
                    log_and_notify_error(f"Failed to load stock data for {stock_code}.")
            except Exception as e:
                logging.error(f"Error processing stock {stock_code}: {e}", exc_info=True)
                continue

        logging.info("Trading process completed.")

        portfolio = {
            'stocks': stock_codes,
            'weights': [1/len(stock_codes)] * len(stock_codes),
            'returns': [0.01] * len(stock_codes)
        }
        portfolio_manager.rebalance_portfolio(portfolio)

        today_str = today.strftime("%Y-%m-%d")
        equity_curve = calculate_equity_curve()  # Define this function to calculate equity curve
        performance_monitor.log_performance(today_str, equity_curve)
        if performance_monitor.trigger_retraining():
            # Implement retraining logic here
            pass
    except Exception as e:
        log_and_notify_error(f"Unhandled exception in main: {e}")

if __name__ == "__main__":
    initialize()
    main()

    while True:
        try:
            # is_trading_day() 함수가 있다면 해당 함수를 사용하여 거래일인지 확인
            if True:  # 예를 들어, is_trading_day()로 바꾸세요.
                schedule.run_pending()
            else:
                logging.info("Today is a holiday. Waiting until the next trading day.")
            time.sleep(60)
        except Exception as e:
            log_and_notify_error(f"Unhandled exception in scheduler loop: {e}")
