import sys
import os
import logging
import threading
import time
from datetime import datetime, timedelta
import schedule
import pandas as pd
from functools import partial

# 프로젝트 루트 디렉토리와 관련된 모든 스크립트 디렉토리를 sys.path에 추가합니다.
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
scripts_dir = os.path.join(project_root, 'scripts')
config_dir = os.path.join(project_root, 'config')
data_dir = os.path.join(project_root, 'data')

sys.path.extend([
    os.path.join(scripts_dir, 'data_collection'),
    os.path.join(scripts_dir, 'model'),
    os.path.join(scripts_dir, 'trading'),
    os.path.join(scripts_dir, 'utils'),
    os.path.join(scripts_dir, 'data_processing'),
    os.path.join(scripts_dir, 'api'),
    config_dir  # config 폴더 경로 추가
])

# config 폴더 경로가 제대로 추가되었는지 확인
print("Config directory added to sys.path:", config_dir in sys.path)
print("Current sys.path:", sys.path)

# 동적으로 config.settings 모듈 로드
import settings

settings = settings.settings

# 필요한 모듈 가져오기
from performance_monitoring import PerformanceMonitor
from portfolio_management import PortfolioManager
from news_sentiment_analysis import NewsSentimentAnalyzer
from error_handler import log_and_notify_error
from websocket_client import connect_to_websocket

logging.basicConfig(level=logging.INFO)

available_cash = 0
performance_monitor = PerformanceMonitor()
portfolio_manager = PortfolioManager()
news_analyzer = NewsSentimentAnalyzer(api_key='YOUR_NEWS_API_KEY')

def update_balance():
    try:
        from check_balance import get_acct_balance
        global available_cash
        _, df_output2, _ = get_acct_balance()
        if df_output2 is not None and not df_output2.empty:
            try:
                dnca_tot_amt = df_output2.iloc[0]['dnca_tot_amt']
                available_cash = float(dnca_tot_amt)
                logging.info(f"Updated available cash: {available_cash}")
            except KeyError as e:
                log_and_notify_error(f"Key error: {e}")
            except Exception as e:
                log_and_notify_error(f"Error updating balance: {e}")
        else:
            log_and_notify_error("Failed to update balance.")
    except Exception as e:
        log_and_notify_error(f"Unhandled exception in update_balance: {e}")

def schedule_tasks():
    schedule.every(1).minute.do(update_balance)
    while True:
        schedule.run_pending()
        time.sleep(1)

def initialize():
    logging.info("Initializing system...")

def update_news_data_for_stock(stock_code):
    from news_data_collector import update_news_data
    logging.info(f"Updating news data for stock code: {stock_code}")
    update_news_data(stock_code)

def get_stock_codes_from_file(file_path):
    try:
        with open(file_path, 'r') as file:
            stock_codes = file.read().splitlines()
        if not stock_codes:
            raise ValueError("No stock codes found in the file.")
        return stock_codes
    except Exception as e:
        log_and_notify_error(f"Error reading stock codes from file: {e}")
        return []

def schedule_stock_news_updates(stock_codes):
    for stock_code in stock_codes:
        logging.info(f"Scheduling update_news_data_for_stock with stock_code: {stock_code}")
        schedule.every().day.at("09:00").do(partial(update_news_data_for_stock, stock_code))
        schedule.every().day.at("15:30").do(partial(update_news_data_for_stock, stock_code))
        schedule.every().day.at("16:00").do(partial(update_news_data_for_stock, stock_code))
        schedule.every(10).minutes.do(partial(update_news_data_for_stock, stock_code))
        schedule.every(1).minutes.do(partial(update_news_data_for_stock, stock_code))

def main():
    try:
        balance_thread = threading.Thread(target=schedule_tasks)
        balance_thread.daemon = True
        balance_thread.start()

        from main_data_collector import collect_all_data
        from stock_data_collector import get_stock_codes, fetch_stock_data, save_data_to_csv
        from data_collection import read_stock_data
        from data_preprocessing import preprocess_data
        from feature_engineering import add_features
        from reinforcement_learning import train_reinforcement_learning_model
        from live_data_processing import real_time_prediction, update_real_time_model
        from trading_logic import execute_trades

        logging.info("Starting data collection...")
        all_data = collect_all_data()
        
        logging.info("Fetching stock codes...")
        global stock_codes  # 전역 변수로 설정하여 다른 함수에서도 접근 가능하도록 함
        stock_codes = get_stock_codes_from_file(os.path.join(data_dir, 'stock_codes.txt'))
        if not stock_codes:
            log_and_notify_error("No stock codes detected. Exiting process.")
            return
        logging.info(f"Stock codes: {stock_codes}")

        today = datetime.now().date()
        ten_years_ago = today - timedelta(days=365 * 10)
        date_range = pd.date_range(end=today, start=ten_years_ago, freq='MS')

        for start_date, end_date in zip(date_range, date_range[1:]):
            logging.info(f"Fetching data for date range: {start_date} to {end_date}")
            for stock_code in stock_codes[:settings['max_detected_stocks']]:
                file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")
                if os.path.exists(file_path):
                    logging.info(f"Data for stock {stock_code} already exists. Skipping.")
                    continue

                data = fetch_stock_data(stock_code, settings['appkey'], settings['appsecret'], access_token, start_date, end_date)
                if data:
                    save_data_to_csv(data, stock_code)
                else:
                    log_and_notify_error(f"No data fetched for stock: {stock_code}")

                time.sleep(0.05)

        logging.info("Data collection completed.")

        for stock_code in stock_codes:
            logging.info(f"Processing stock: {stock_code}")
            stock_data = read_stock_data(stock_code)
            if stock_data is not None:
                logging.info(f"Stock data for {stock_code} loaded successfully.")
                
                (train_data, test_data), scaler = preprocess_data(stock_data, None, None, None)
                stock_data = add_features(stock_data)

                rl_model = train_reinforcement_learning_model(stock_data)

                real_time_predictions = real_time_prediction(rl_model, scaler, stock_code)
                if real_time_predictions is not None:
                    logging.info(f"Real-time predictions for {stock_code}: {real_time_predictions}")
                else:
                    log_and_notify_error("Failed to fetch real-time predictions.")

                model = update_real_time_model(rl_model, scaler, stock_code)

                sentiment = news_analyzer.get_stock_sentiment(stock_code)
                logging.info(f"News sentiment for {stock_code}: {sentiment}")

                execute_trades(stock_data)
            else:
                log_and_notify_error(f"Failed to load stock data for {stock_code}.")

        logging.info("Trading process completed.")

        portfolio = {
            'stocks': stock_codes,
            'weights': [1/len(stock_codes)] * len(stock_codes),
            'returns': [0.01] * len(stock_codes)
        }
        portfolio_manager.rebalance_portfolio(portfolio)

        today_str = today.strftime("%Y-%m-%d")
        equity_curve = calculate_equity_curve()  # Define this function to calculate equity curve
        performance_monitor.log_performance(today_str, equity_curve)
        if performance_monitor.trigger_retraining():
            # Implement retraining logic here
            pass
    except Exception as e:
        log_and_notify_error(f"Unhandled exception in main: {e}")

if __name__ == "__main__":
    initialize()
    main()

    stock_codes = get_stock_codes_from_file(os.path.join(data_dir, 'stock_codes.txt'))
    if stock_codes:
        schedule_stock_news_updates(stock_codes)

    while True:
        try:
            if True:  # is_trading_day() 함수가 있는 경우 그 함수로 바꾸세요.
                schedule.run_pending()
            else:
                logging.info("Today is a holiday. Waiting until the next trading day.")
            time.sleep(60)
        except Exception as e:
            log_and_notify_error(f"Unhandled exception in scheduler loop: {e}")
